@@ -19,6 +19,8 @@ class Challenger(ChallengerInterface):
     is later used by the simple_cookie credentials extractor.
     """
 
+    desc = "TiddlyWeb username and password"
+
     def challenge_get(self, environ, start_response):
         """
         Respond to a GET request by sending a form.
@@ -44,10 +46,10 @@ class Challenger(ChallengerInterface):
                     user, password, redirect)
         except KeyError:
             return self._send_cookie_form(environ, start_response,
-                    redirect, '401 Unauthorized')
+                    redirect, message='missing input')
 
     def _send_cookie_form(self, environ, start_response, redirect,
-            status='200 OK', message=''):
+            status='401 Unauthorized', message=''):
         """
         Send a simple form to the client asking for a username
         and password.
@@ -60,7 +62,7 @@ class Challenger(ChallengerInterface):
 %s
 <form action="" method="POST">
 User: <input name="user" size="40" />
-Password <input type="password" name="password" size="40" />
+Password: <input type="password" name="password" size="40" />
 <input type="hidden" name="tiddlyweb_redirect" value="%s" />
 <input type="submit" value="submit" />
 </form>
